package de.ab23.dao.billing;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import javax.transaction.UserTransaction;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.EmptyAsset;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import de.ab23.billing.domain.Game;
import de.ab23.common.producers.OtherExampleProducer;
import de.ab23.common.qualifiers.OtherExampleQual;
import de.ab23.dao.billing.impl.OtherExampleDao;

/**
 * @author <a href="http://community.jboss.org/people/dan.j.allen">Dan Allen</a>
 */
@RunWith(Arquillian.class)
public class GamePersistenceTest {
	@Deployment
	public static Archive<?> createDeployment() {

		WebArchive war = ShrinkWrap.create(WebArchive.class, "test.war").addClass(Game.class).addClass(OtherExampleDao.class)
				.addClass(OtherExampleDaoIf.class).addClass(OtherExampleProducer.class).addClass(OtherExampleQual.class)
				.addAsResource("test-persistence.xml", "META-INF/persistence.xml").addAsWebInfResource("jbossas-ds.xml")
				.addAsWebInfResource(EmptyAsset.INSTANCE, "beans.xml");

		//System.out.println(war.as(WebArchive.class).toString(true));
		//war.as(ZipExporter.class).exportTo(new File("c:/Daten/test.war"), true);
		return war;
	}

	private static final String[] GAME_TITLES = { "Super Mario Brothers", "Mario Kart", "F-Zero" };

	private static final String[] NEW_GAME_TITLES = { "Ring", "Bell" };

	@PersistenceContext
	EntityManager em;

	@Inject
	UserTransaction utx;

	@Inject
	OtherExampleDaoIf dao;

	@Before
	public void preparePersistenceTest() throws Exception {
		clearData();
		insertData();
		startTransaction();
	}

	private void clearData() throws Exception {
		utx.begin();
		em.joinTransaction();
		System.out.println("Dumping old records...");
		em.createQuery("delete from Game").executeUpdate();
		utx.commit();
	}

	private void insertData() throws Exception {
		utx.begin();
		em.joinTransaction();
		System.out.println("Inserting records...");
		for (String title : GAME_TITLES) {
			Game game = new Game(title);
			em.persist(game);
		}
		utx.commit();
		// reset the persistence context (cache)
		em.clear();
	}

	private void startTransaction() throws Exception {
		utx.begin();
		em.joinTransaction();
	}

	@After
	public void commitTransaction() throws Exception {
		utx.commit();
	}

	@Test
	public void shouldFindAllGamesUsingJpqlQuery() throws Exception {
		// given
		String fetchingAllGamesInJpql = "select g from Game g order by g.id";

		// when
		System.out.println("Selecting (using JPQL)...");
		List<Game> games = em.createQuery(fetchingAllGamesInJpql, Game.class).getResultList();

		// then
		System.out.println("Found " + games.size() + " games (using JPQL):");
		assertContainsAllGames(games);
	}

	@Test
	public void shouldFindAllGamesUsingCriteriaApi() throws Exception {
		// given
		CriteriaBuilder builder = em.getCriteriaBuilder();
		CriteriaQuery<Game> criteria = builder.createQuery(Game.class);

		Root<Game> game = criteria.from(Game.class);
		criteria.select(game);
		// TIP: If you don't want to use the JPA 2 Metamodel,
		// you can change the get() method call to get("id")
		criteria.orderBy(builder.asc(game.get("id")));
		// No WHERE clause, which implies select all

		// when
		System.out.println("Selecting (using Criteria)...");
		List<Game> games = em.createQuery(criteria).getResultList();

		// then
		System.out.println("Found " + games.size() + " games (using Criteria):");
		assertContainsAllGames(games);
	}

	@Test
	public void createMyGames() {
		dao.persistGames(NEW_GAME_TITLES);
		List<Game> persistedGames = dao.findAll();
		for (Game game : persistedGames) {
			System.out.println("Found: " + game);
		}
		Assert.assertEquals(NEW_GAME_TITLES.length + GAME_TITLES.length, persistedGames.size());
	}

	private static void assertContainsAllGames(Collection<Game> retrievedGames) {
		Assert.assertEquals(GAME_TITLES.length, retrievedGames.size());
		final Set<String> retrievedGameTitles = new HashSet<String>();
		for (Game game : retrievedGames) {
			System.out.println("* " + game);
			retrievedGameTitles.add(game.getTitle());
		}
		Assert.assertTrue(retrievedGameTitles.containsAll(Arrays.asList(GAME_TITLES)));
	}
}
